"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
Object.defineProperty(exports, "__esModule", { value: true });
const imperative_1 = require("@zowe/imperative");
/**
 * Utility Methods for Brightside
 * @export
 */
class ZosmfSession {
    /**
     * Given a z/OSMF profile, create a REST Client Session.
     * @static
     * @param {IProfile} profile - The z/OSMF profile contents
     * @returns {Session} - A session for usage in the z/OSMF REST Client
     */
    static createBasicZosmfSession(profile) {
        this.log.debug("Creating a z/OSMF session from the profile named %s", profile.name);
        return new imperative_1.Session({
            type: "basic",
            hostname: profile.host,
            port: profile.port,
            user: profile.user,
            password: profile.password,
            base64EncodedAuth: profile.auth,
            rejectUnauthorized: profile.rejectUnauthorized,
            basePath: profile.basePath
        });
    }
    /**
     * Given command line arguments, create a REST Client Session.
     * @static
     * @param {IProfile} args - The arguments specified by the user
     * @returns {Session} - A session for usage in the z/OSMF REST Client
     */
    static createBasicZosmfSessionFromArguments(args) {
        this.log.debug("Creating a z/OSMF session from arguments");
        return new imperative_1.Session({
            type: "basic",
            hostname: args.host,
            port: args.port,
            user: args.user,
            password: args.password,
            base64EncodedAuth: args.auth,
            rejectUnauthorized: args.rejectUnauthorized,
            basePath: args.basePath
        });
    }
    static get log() {
        return imperative_1.Logger.getAppLogger();
    }
}
exports.ZosmfSession = ZosmfSession;
ZosmfSession.ZOSMF_CONNECTION_OPTION_GROUP = "Zosmf Connection Options";
/**
 * Option used in profile creation and commands for hostname for z/OSMF
 */
ZosmfSession.ZOSMF_OPTION_HOST = {
    name: "host",
    aliases: ["H"],
    description: "The z/OSMF server host name.",
    type: "string",
    required: true,
    group: ZosmfSession.ZOSMF_CONNECTION_OPTION_GROUP
};
/**
 * Option used in profile creation and commands for port for z/OSMF
 */
ZosmfSession.ZOSMF_OPTION_PORT = {
    name: "port",
    aliases: ["P"],
    description: "The z/OSMF server port.",
    type: "number",
    defaultValue: 443,
    group: ZosmfSession.ZOSMF_CONNECTION_OPTION_GROUP
};
/**
 * Option used in profile creation and commands for username / ID  for z/OSMF
 */
ZosmfSession.ZOSMF_OPTION_USER = {
    name: "user",
    aliases: ["u"],
    description: "Mainframe (z/OSMF) user name, which can be the same as your TSO login.",
    type: "string",
    required: true,
    group: ZosmfSession.ZOSMF_CONNECTION_OPTION_GROUP
};
/**
 * Option used in profile creation and commands for password/passphrase for z/OSMF
 */
ZosmfSession.ZOSMF_OPTION_PASSWORD = {
    name: "password",
    aliases: ["pass", "pw"],
    description: "Mainframe (z/OSMF) password, which can be the same as your TSO password.",
    type: "string",
    group: ZosmfSession.ZOSMF_CONNECTION_OPTION_GROUP,
    required: true
};
/**
 * Option used in profile creation and commands for rejectUnauthorized setting for connecting to z/OSMF
 */
ZosmfSession.ZOSMF_OPTION_REJECT_UNAUTHORIZED = {
    name: "reject-unauthorized",
    aliases: ["ru"],
    description: "Reject self-signed certificates.",
    type: "boolean",
    defaultValue: true,
    group: ZosmfSession.ZOSMF_CONNECTION_OPTION_GROUP
};
/**
 * Option used in profile creation and commands for base path setting for connecting to z/OSMF
 */
ZosmfSession.ZOSMF_OPTION_BASE_PATH = {
    name: "base-path",
    aliases: ["bp"],
    description: "The base path for your API mediation layer instance." +
        " Specify this option to prepend the base path to all z/OSMF resources when making REST requests." +
        " Do not specify this option if you are not using an API mediation layer.",
    type: "string",
    group: ZosmfSession.ZOSMF_CONNECTION_OPTION_GROUP
};
/**
 * Options related to connecting to z/OSMF
 * These options can be filled in if the user creates a profile
 */
ZosmfSession.ZOSMF_CONNECTION_OPTIONS = [
    ZosmfSession.ZOSMF_OPTION_HOST,
    ZosmfSession.ZOSMF_OPTION_PORT,
    ZosmfSession.ZOSMF_OPTION_USER,
    ZosmfSession.ZOSMF_OPTION_PASSWORD,
    ZosmfSession.ZOSMF_OPTION_REJECT_UNAUTHORIZED,
    ZosmfSession.ZOSMF_OPTION_BASE_PATH
];
//# sourceMappingURL=ZosmfSession.js.map