"use strict";
/*!
 * This program and the accompanying materials are made available under the terms of the
 * Eclipse Public License v2.0 which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-v20.html
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Copyright Contributors to the Zowe Project.
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../constants");
const environment_1 = require("../environment");
const fs = require("fs-extra");
const os = require("os");
/**
 * Environment key prefix for the max history value. Max history refers to the
 * max number of log entries to keep when performance is enabled.
 *
 * @external
 */
const ENV_IO_MAX_HISTORY = `${constants_1.ENV_PREFIX}_IO_MAX_HISTORY`;
/**
 * Environment key prefix for where the logs are saved.
 *
 * @external
 */
const ENV_IO_SAVE_DIR = `${constants_1.ENV_PREFIX}_IO_SAVE_DIR`;
// tslint:disable:no-magic-numbers
environment_1.Environment
    .register(ENV_IO_MAX_HISTORY, 5)
    .register(ENV_IO_SAVE_DIR, `${os.homedir()}/.perf-timing`);
// tslint:enable:no-magic-numbers
/**
 * Save metrics into the next log file.
 *
 * Upon execution this function will first roll any logs based on the value
 * present within environmental {@link ENV_IO_MAX_HISTORY} variable. The directory
 * that is checked will be determined from the {@link ENV_IO_SAVE_DIR} value.
 *
 * @param data The object data to save in a file.
 *
 * @todo This function needs to be enhanced so that an environment variable can
 * influence the file name.
 *
 * @internal
 */
function saveMetrics(data) {
    // @FUTURE This is where file save location will be determined by
    // @FUTURE environment settings for aggregation at a later time.
    const directory = environment_1.Environment.getValue(ENV_IO_SAVE_DIR);
    fs.ensureDirSync(directory);
    // Prior to saving roll any metrics beforehand
    // Get the starting index
    let historyIndex = environment_1.Environment.getNumber(ENV_IO_MAX_HISTORY);
    const lastFile = getMetricFileName(directory, historyIndex);
    // Delete the last file when history limit has been reached
    fs.removeSync(lastFile);
    // Move all historical logs down by 1
    while (--historyIndex > 0) {
        const fileName = getMetricFileName(directory, historyIndex);
        if (fs.existsSync(fileName)) {
            fs.renameSync(fileName, getMetricFileName(directory, historyIndex + 1));
        }
    }
    // Write the first metric
    fs.writeFileSync(getMetricFileName(directory, 1), JSON.stringify(data));
}
exports.saveMetrics = saveMetrics;
/**
 * Get a constant file path for a given history item index.
 *
 * @param directory The directory of the file
 * @param index The index of the file in the history
 *
 * @returns The formatted file name that will be saved
 *
 * @internal
 */
function getMetricFileName(directory, index) {
    return `${directory}/metrics.${index}.json`;
}
//# sourceMappingURL=saveMetrics.js.map