"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const imperative_1 = require("@zowe/imperative");
const path = require("path");
const url_1 = require("url");
const vscode = require("vscode");
const zowe = require("@zowe/cli");
const ZoweExplorerApiRegister_1 = require("./api/ZoweExplorerApiRegister");
const nls = require("vscode-nls");
const utils_1 = require("./utils");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
let IConnection;
class Profiles {
    constructor(log) {
        this.log = log;
        this.allProfiles = [];
        this.validProfile = -1;
        this.profilesByType = new Map();
        this.defaultProfileByType = new Map();
        this.profileManagerByType = new Map();
    }
    // Processing stops if there are no profiles detected
    static createInstance(log) {
        return __awaiter(this, void 0, void 0, function* () {
            Profiles.loader = new Profiles(log);
            yield Profiles.loader.refresh();
            return Profiles.loader;
        });
    }
    static getInstance() {
        return Profiles.loader;
    }
    checkCurrentProfile(theProfile) {
        return __awaiter(this, void 0, void 0, function* () {
            if ((!theProfile.profile.user) || (!theProfile.profile.password)) {
                try {
                    const values = yield Profiles.getInstance().promptCredentials(theProfile.name);
                    if (values !== undefined) {
                        this.usrNme = values[0];
                        this.passWrd = values[1];
                        this.baseEncd = values[2];
                    }
                }
                catch (error) {
                    utils_1.errorHandling(error, theProfile.name, localize(0, null) + `createUSSNodeDialog.optionalProfiles!`);
                    return;
                }
                if (this.usrNme !== undefined && this.passWrd !== undefined && this.baseEncd !== undefined) {
                    theProfile.profile.user = this.usrNme;
                    theProfile.profile.password = this.passWrd;
                    theProfile.profile.base64EncodedAuth = this.baseEncd;
                    this.validProfile = 0;
                }
                else {
                    return;
                }
            }
            else {
                this.validProfile = 0;
            }
        });
    }
    loadNamedProfile(name, type) {
        for (const profile of this.allProfiles) {
            if (profile.name === name && (type ? profile.type === type : true)) {
                return profile;
            }
        }
        throw new Error(localize(1, null)
            + name + localize(2, null));
    }
    getDefaultProfile(type = "zosmf") {
        return this.defaultProfileByType.get(type);
    }
    getProfiles(type = "zosmf") {
        return this.profilesByType.get(type);
    }
    refresh() {
        return __awaiter(this, void 0, void 0, function* () {
            this.allProfiles = [];
            for (const type of ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getInstance().registeredApiTypes()) {
                const profileManager = yield this.getCliProfileManager(type);
                const profilesForType = (yield profileManager.loadAll()).filter((profile) => {
                    return profile.type === type;
                });
                if (profilesForType && profilesForType.length > 0) {
                    this.allProfiles.push(...profilesForType);
                    this.profilesByType.set(type, profilesForType);
                    this.defaultProfileByType.set(type, (yield profileManager.load({ loadDefault: true })));
                }
            }
        });
    }
    validateAndParseUrl(newUrl) {
        let url;
        const validProtocols = ["https"];
        const DEFAULT_HTTPS_PORT = 443;
        const validationResult = {
            valid: false,
            host: null,
            port: null
        };
        try {
            url = new url_1.URL(newUrl);
        }
        catch (error) {
            return validationResult;
        }
        // overkill with only one valid protocol, but we may expand profile types and protocols in the future?
        if (!validProtocols.some((validProtocol) => url.protocol.includes(validProtocol))) {
            return validationResult;
        }
        // if port is empty, set https defaults
        if (!url.port.trim()) {
            validationResult.port = DEFAULT_HTTPS_PORT;
        }
        else {
            validationResult.port = Number(url.port);
        }
        validationResult.host = url.hostname;
        validationResult.valid = true;
        return validationResult;
    }
    getUrl(urlInputBox) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => {
                urlInputBox.onDidHide(() => { resolve(urlInputBox.value); });
                urlInputBox.onDidAccept(() => {
                    if (this.validateAndParseUrl(urlInputBox.value).valid) {
                        resolve(urlInputBox.value);
                    }
                    else {
                        urlInputBox.validationMessage = localize(3, null);
                    }
                });
            });
        });
    }
    createNewConnection(profileName, profileType = "zosmf") {
        return __awaiter(this, void 0, void 0, function* () {
            let userName;
            let passWord;
            let zosmfURL;
            let rejectUnauthorize;
            let options;
            const urlInputBox = vscode.window.createInputBox();
            urlInputBox.ignoreFocusOut = true;
            urlInputBox.placeholder = localize(4, null);
            urlInputBox.prompt = localize(5, null);
            urlInputBox.show();
            zosmfURL = yield this.getUrl(urlInputBox);
            urlInputBox.dispose();
            if (!zosmfURL) {
                vscode.window.showInformationMessage(localize(6, null));
                return undefined;
            }
            const zosmfUrlParsed = this.validateAndParseUrl(zosmfURL);
            options = {
                placeHolder: localize(7, null),
                prompt: localize(8, null),
                value: userName
            };
            userName = yield vscode.window.showInputBox(options);
            if (userName === undefined) {
                vscode.window.showInformationMessage(localize(9, null));
                return;
            }
            options = {
                placeHolder: localize(10, null),
                prompt: localize(11, null),
                password: true,
                value: passWord
            };
            passWord = yield vscode.window.showInputBox(options);
            if (passWord === undefined) {
                vscode.window.showInformationMessage(localize(12, null));
                return;
            }
            const quickPickOptions = {
                placeHolder: localize(13, null),
                ignoreFocusOut: true,
                canPickMany: false
            };
            const selectRU = ["True - Reject connections with self-signed certificates",
                "False - Accept connections with self-signed certificates"];
            const ruOptions = Array.from(selectRU);
            const chosenRU = yield vscode.window.showQuickPick(ruOptions, quickPickOptions);
            if (chosenRU === ruOptions[0]) {
                rejectUnauthorize = true;
            }
            else if (chosenRU === ruOptions[1]) {
                rejectUnauthorize = false;
            }
            else {
                vscode.window.showInformationMessage(localize(14, null));
                return undefined;
            }
            for (const profile of this.allProfiles) {
                if (profile.name === profileName) {
                    vscode.window.showErrorMessage(localize(15, null));
                    return undefined;
                }
            }
            IConnection = {
                name: profileName,
                host: zosmfUrlParsed.host,
                port: zosmfUrlParsed.port,
                user: userName,
                password: passWord,
                rejectUnauthorized: rejectUnauthorize
            };
            let newProfile;
            try {
                newProfile = yield this.saveProfile(IConnection, IConnection.name, profileType);
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message);
            }
            yield zowe.ZosmfSession.createBasicZosmfSession(newProfile);
            vscode.window.showInformationMessage("Profile " + profileName + " was created.");
            return profileName;
        });
    }
    promptCredentials(sessName, rePrompt) {
        return __awaiter(this, void 0, void 0, function* () {
            let userName;
            let passWord;
            let options;
            const loadProfile = this.loadNamedProfile(sessName.trim());
            const loadSession = loadProfile.profile;
            if (rePrompt) {
                userName = loadSession.user;
                passWord = loadSession.password;
            }
            if (!loadSession.user || rePrompt) {
                options = {
                    placeHolder: localize(16, null),
                    prompt: localize(17, null),
                    value: userName
                };
                userName = yield vscode.window.showInputBox(options);
                if (!userName) {
                    vscode.window.showErrorMessage(localize(18, null));
                    return;
                }
                else {
                    loadSession.user = loadProfile.profile.user = userName;
                }
            }
            if (!loadSession.password || rePrompt) {
                passWord = loadSession.password;
                options = {
                    placeHolder: localize(19, null),
                    prompt: localize(20, null),
                    password: true,
                    value: passWord
                };
                passWord = yield vscode.window.showInputBox(options);
                if (!passWord) {
                    vscode.window.showErrorMessage(localize(21, null));
                    return;
                }
                else {
                    loadSession.password = loadProfile.profile.password = passWord.trim();
                }
            }
            const updSession = yield zowe.ZosmfSession.createBasicZosmfSession(loadSession);
            if (rePrompt) {
                yield this.updateProfile(loadProfile);
            }
            return [updSession.ISession.user, updSession.ISession.password, updSession.ISession.base64EncodedAuth];
        });
    }
    updateProfile(ProfileInfo) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const type of ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getInstance().registeredApiTypes()) {
                const profileManager = yield this.getCliProfileManager(type);
                this.loadedProfile = (yield profileManager.load({ name: ProfileInfo.name }));
            }
            const OrigProfileInfo = this.loadedProfile.profile;
            const NewProfileInfo = ProfileInfo.profile;
            if (OrigProfileInfo.user) {
                OrigProfileInfo.user = NewProfileInfo.user;
            }
            if (OrigProfileInfo.password) {
                OrigProfileInfo.password = NewProfileInfo.password;
            }
            const updateParms = {
                name: this.loadedProfile.name,
                merge: true,
                profile: OrigProfileInfo
            };
            try {
                (yield this.getCliProfileManager(this.loadedProfile.type)).update(updateParms);
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message);
            }
        });
    }
    saveProfile(ProfileInfo, ProfileName, ProfileType) {
        return __awaiter(this, void 0, void 0, function* () {
            let zosmfProfile;
            try {
                zosmfProfile = yield (yield this.getCliProfileManager(ProfileType)).save({ profile: ProfileInfo, name: ProfileName, type: ProfileType });
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message);
            }
            return zosmfProfile.profile;
        });
    }
    getCliProfileManager(type) {
        return __awaiter(this, void 0, void 0, function* () {
            let profileManager = this.profileManagerByType.get(type);
            if (!profileManager) {
                profileManager = yield new imperative_1.CliProfileManager({
                    profileRootDirectory: path.join(utils_1.getZoweDir(), "profiles"),
                    type
                });
                if (profileManager) {
                    this.profileManagerByType.set(type, profileManager);
                }
                else {
                    return undefined;
                }
            }
            return profileManager;
        });
    }
}
exports.Profiles = Profiles;

//# sourceMappingURL=Profiles.js.map
