"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const globals = require("../globals");
const dsActions = require("./actions");
const Profiles_1 = require("../Profiles");
const ZoweExplorerApiRegister_1 = require("../api/ZoweExplorerApiRegister");
const utils_1 = require("../utils");
const utils_2 = require("../shared/utils");
const ZoweTreeProvider_1 = require("../abstract/ZoweTreeProvider");
const ZoweDatasetNode_1 = require("./ZoweDatasetNode");
const icons_1 = require("../generators/icons");
const fs = require("fs");
const nls = require("vscode-nls");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
/**
 * Creates the Dataset tree that contains nodes of sessions and data sets
 *
 * @export
 */
function createDatasetTree(log) {
    return __awaiter(this, void 0, void 0, function* () {
        const tree = new DatasetTree();
        yield tree.initialize(log);
        yield tree.addSession();
        return tree;
    });
}
exports.createDatasetTree = createDatasetTree;
/**
 * A tree that contains nodes of sessions and data sets
 *
 * @export
 * @class DatasetTree
 * @implements {vscode.TreeDataProvider}
 */
class DatasetTree extends ZoweTreeProvider_1.ZoweTreeProvider {
    constructor() {
        super(DatasetTree.persistenceSchema, new ZoweDatasetNode_1.ZoweDatasetNode(localize(0, null), vscode.TreeItemCollapsibleState.Collapsed, null, null, null));
        this.mSessionNodes = [];
        this.mFavorites = [];
        this.mFavoriteSession.contextValue = globals.FAVORITE_CONTEXT;
        const icon = icons_1.getIconByNode(this.mFavoriteSession);
        if (icon) {
            this.mFavoriteSession.iconPath = icon.path;
        }
        this.mSessionNodes = [this.mFavoriteSession];
        this.treeView = vscode.window.createTreeView("zowe.explorer", { treeDataProvider: this });
    }
    /**
     * Rename data set
     *
     * @export
     * @param node - The node
     */
    rename(node) {
        return __awaiter(this, void 0, void 0, function* () {
            switch (node.contextValue) {
                case globals.DS_MEMBER_CONTEXT:
                case (globals.DS_MEMBER_CONTEXT + globals.FAV_SUFFIX):
                    return this.renameDataSetMember(node);
                default:
                    return this.renameDataSet(node);
            }
        });
    }
    open(node, preview) {
        throw new Error("Method not implemented.");
    }
    copy(node) {
        throw new Error("Method not implemented.");
    }
    paste(node) {
        throw new Error("Method not implemented.");
    }
    delete(node) {
        throw new Error("Method not implemented.");
    }
    saveSearch(node) {
        throw new Error("Method not implemented.");
    }
    saveFile(document) {
        throw new Error("Method not implemented.");
    }
    refreshPS(node) {
        throw new Error("Method not implemented.");
    }
    uploadDialog(node) {
        throw new Error("Method not implemented.");
    }
    filterPrompt(node) {
        return this.datasetFilterPrompt(node);
    }
    /**
     * Takes argument of type IZoweDatasetTreeNode and retrieves all of the first level children
     *
     * @param [element] - Optional parameter; if not passed, returns root session nodes
     * @returns {IZoweDatasetTreeNode[] | Promise<IZoweDatasetTreeNode[]>}
     */
    getChildren(element) {
        return __awaiter(this, void 0, void 0, function* () {
            if (element) {
                if (element.contextValue === globals.FAVORITE_CONTEXT) {
                    return this.mFavorites;
                }
                yield Profiles_1.Profiles.getInstance().checkCurrentProfile(element.getProfile());
                return element.getChildren();
            }
            return this.mSessionNodes;
        });
    }
    /**
     * Initializes the tree based on favorites held in persistent store
     *
     * @param log
     */
    initialize(log) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log = log;
            this.log.debug(localize(1, null));
            const lines = this.mHistory.readFavorites();
            for (const line of lines) {
                if (line === "") {
                    continue;
                }
                // validate line
                const favoriteDataSetPattern = /^\[.+\]\:\s[a-zA-Z#@\$][a-zA-Z0-9#@\$\-]{0,7}(\.[a-zA-Z#@\$][a-zA-Z0-9#@\$\-]{0,7})*\{p?ds\}$/;
                const favoriteSearchPattern = /^\[.+\]\:\s.*\{session\}$/;
                if (favoriteDataSetPattern.test(line)) {
                    const sesName = line.substring(1, line.lastIndexOf("]")).trim();
                    try {
                        const profile = Profiles_1.Profiles.getInstance().loadNamedProfile(sesName);
                        const session = ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(profile).getSession();
                        let node;
                        if (line.substring(line.indexOf("{") + 1, line.lastIndexOf("}")) === globals.DS_PDS_CONTEXT) {
                            node = new ZoweDatasetNode_1.ZoweDatasetNode(line.substring(0, line.indexOf("{")), vscode.TreeItemCollapsibleState.Collapsed, this.mFavoriteSession, session, undefined, undefined, profile);
                        }
                        else {
                            node = new ZoweDatasetNode_1.ZoweDatasetNode(line.substring(0, line.indexOf("{")), vscode.TreeItemCollapsibleState.None, this.mFavoriteSession, session, undefined, undefined, profile);
                            node.command = { command: "zowe.ZoweNode.openPS", title: "", arguments: [node] };
                        }
                        node.contextValue += globals.FAV_SUFFIX;
                        const icon = icons_1.getIconByNode(node);
                        if (icon) {
                            node.iconPath = icon.path;
                        }
                        this.mFavorites.push(node);
                    }
                    catch (e) {
                        const errMessage = localize(2, null) + sesName +
                            localize(3, null) +
                            localize(4, null) +
                            utils_2.getAppName(globals.ISTHEIA) + localize(5, null);
                        yield utils_1.errorHandling(e, null, errMessage);
                        continue;
                    }
                }
                else if (favoriteSearchPattern.test(line)) {
                    const sesName = line.substring(1, line.lastIndexOf("]")).trim();
                    let profile;
                    try {
                        profile = Profiles_1.Profiles.getInstance().loadNamedProfile(sesName);
                    }
                    catch (error) {
                        const errMessage = localize(6, null) +
                            +sesName +
                            localize(7, null);
                        yield utils_1.errorHandling(error, null, errMessage);
                        continue;
                    }
                    const session = ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(profile).getSession();
                    const node = new ZoweDatasetNode_1.ZoweDatasetNode(line.substring(0, line.lastIndexOf("{")), vscode.TreeItemCollapsibleState.None, this.mFavoriteSession, session, undefined, undefined, profile);
                    node.command = { command: "zowe.pattern", title: "", arguments: [node] };
                    node.contextValue = globals.DS_SESSION_CONTEXT + globals.FAV_SUFFIX;
                    const icon = icons_1.getIconByNode(node);
                    if (icon) {
                        node.iconPath = icon.path;
                    }
                    this.mFavorites.push(node);
                }
                else {
                    vscode.window.showErrorMessage(localize(8, null) + line);
                }
            }
        });
    }
    /**
     * Returns the tree view for the current DatasetTree
     *
     * @returns {vscode.TreeView<IZoweDatasetTreeNode>}
     */
    getTreeView() {
        return this.treeView;
    }
    /**
     * Adds a new session to the data set tree
     *
     * @param {string} [sessionName] - optional; loads default profile if not passed
     */
    addSession(sessionName) {
        return __awaiter(this, void 0, void 0, function* () {
            // Loads profile associated with passed sessionName, default if none passed
            if (sessionName) {
                const zosmfProfile = Profiles_1.Profiles.getInstance().loadNamedProfile(sessionName);
                if (zosmfProfile) {
                    this.addSingleSession(zosmfProfile);
                }
            }
            else {
                const profiles = Profiles_1.Profiles.getInstance().allProfiles;
                for (const zosmfProfile of profiles) {
                    // If session is already added, do nothing
                    if (this.mSessionNodes.find((tempNode) => tempNode.label.trim() === zosmfProfile.name)) {
                        continue;
                    }
                    for (const session of this.mHistory.getSessions()) {
                        if (session === zosmfProfile.name) {
                            this.addSingleSession(zosmfProfile);
                        }
                    }
                }
                if (this.mSessionNodes.length === 1) {
                    this.addSingleSession(Profiles_1.Profiles.getInstance().getDefaultProfile());
                }
            }
            this.refresh();
        });
    }
    /**
     * Removes a session from the list in the data set tree
     *
     * @param node
     */
    deleteSession(node) {
        this.mSessionNodes = this.mSessionNodes.filter((tempNode) => tempNode.label.trim() !== node.label.trim());
        let revisedLabel = node.label;
        if (revisedLabel.includes("[")) {
            revisedLabel = revisedLabel.substring(0, revisedLabel.indexOf(" ["));
        }
        this.mHistory.removeSession(revisedLabel);
        this.refresh();
    }
    /**
     * Adds a node to the favorites list
     *
     * @param  node
     */
    addFavorite(node) {
        return __awaiter(this, void 0, void 0, function* () {
            let temp;
            if (node.contextValue === globals.DS_MEMBER_CONTEXT) {
                if (node.getParent().contextValue === globals.DS_PDS_CONTEXT + globals.FAV_SUFFIX) {
                    vscode.window.showInformationMessage(localize(9, null));
                    return;
                }
                this.addFavorite(node.getParent());
                return;
            }
            else if (node.contextValue === globals.DS_SESSION_CONTEXT) {
                temp = new ZoweDatasetNode_1.ZoweDatasetNode("[" + node.getSessionNode().label.trim() + "]: " + node.pattern, vscode.TreeItemCollapsibleState.None, this.mFavoriteSession, node.getSession(), node.contextValue, node.getEtag(), node.getProfile());
                temp.contextValue = globals.DS_SESSION_CONTEXT + globals.FAV_SUFFIX;
                const icon = icons_1.getIconByNode(temp);
                if (icon) {
                    temp.iconPath = icon.path;
                }
                // add a command to execute the search
                temp.command = { command: "zowe.pattern", title: "", arguments: [temp] };
            }
            else { // pds | ds
                temp = new ZoweDatasetNode_1.ZoweDatasetNode("[" + node.getSessionNode().label.trim() + "]: " + node.label, node.collapsibleState, this.mFavoriteSession, node.getSession(), node.contextValue, node.getEtag(), node.getProfile());
                temp.contextValue += globals.FAV_SUFFIX;
                if (temp.contextValue === globals.DS_DS_CONTEXT + globals.FAV_SUFFIX) {
                    temp.command = { command: "zowe.ZoweNode.openPS", title: "", arguments: [temp] };
                }
                const icon = icons_1.getIconByNode(temp);
                if (icon) {
                    temp.iconPath = icon.path;
                }
            }
            if (!this.mFavorites.find((tempNode) => (tempNode.label === temp.label) && (tempNode.contextValue === temp.contextValue))) {
                this.mFavorites.push(temp);
                utils_2.sortTreeItems(this.mFavorites, globals.DS_SESSION_CONTEXT + globals.FAV_SUFFIX);
                yield this.updateFavorites();
                this.refreshElement(this.mFavoriteSession);
            }
        });
    }
    /**
     * Renames a node based on the profile and it's label
     *
     * @param profileLabel
     * @param beforeLabel
     * @param afterLabel
     */
    renameNode(profileLabel, beforeLabel, afterLabel) {
        return __awaiter(this, void 0, void 0, function* () {
            const sessionNode = this.mSessionNodes.find((session) => session.label.trim() === profileLabel.trim());
            if (sessionNode) {
                const matchingNode = sessionNode.children.find((node) => node.label === beforeLabel);
                if (matchingNode) {
                    matchingNode.label = afterLabel;
                    this.refreshElement(matchingNode);
                }
            }
        });
    }
    /**
     * Renames a node from the favorites list
     *
     * @param node
     */
    renameFavorite(node, newLabel) {
        return __awaiter(this, void 0, void 0, function* () {
            const matchingNode = this.mFavorites.find((temp) => (temp.label === node.label) && (temp.contextValue.startsWith(node.contextValue)));
            if (matchingNode) {
                const prefix = matchingNode.label.substring(0, matchingNode.label.indexOf(":") + 2);
                matchingNode.label = prefix + newLabel;
                this.refreshElement(matchingNode);
            }
        });
    }
    /**
     * Finds the equivalent node as a favorite
     *
     * @param node
     */
    findFavoritedNode(node) {
        return this.mFavorites.find((temp) => (temp.label === `[${node.getParent().getLabel()}]: ${node.label}`) && (temp.contextValue.includes(node.contextValue)));
    }
    /**
     * Finds the equivalent node not as a favorite
     *
     * @param node
     */
    findNonFavoritedNode(node) {
        const profileLabel = node.label.substring(1, node.label.indexOf("]"));
        const nodeLabel = node.label.substring(node.label.indexOf(":") + 2);
        const sessionNode = this.mSessionNodes.find((session) => session.label.trim() === profileLabel);
        return sessionNode.children.find((temp) => temp.label === nodeLabel);
    }
    /**
     * Removes a node from the favorites list
     *
     * @param node
     */
    removeFavorite(node) {
        return __awaiter(this, void 0, void 0, function* () {
            this.mFavorites = this.mFavorites.filter((temp) => !((temp.label === node.label) && (temp.contextValue.startsWith(node.contextValue))));
            this.refresh();
            yield this.updateFavorites();
            this.refreshElement(this.mFavoriteSession);
        });
    }
    updateFavorites() {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = this.mFavorites.map((fav) => fav.label + "{" + fav.contextValue.substring(0, fav.contextValue.indexOf(globals.FAV_SUFFIX)) + "}");
            this.mHistory.updateFavorites(settings);
        });
    }
    onDidChangeConfiguration(e) {
        return __awaiter(this, void 0, void 0, function* () {
            // Empties the persistent favorites & history arrays, if the user has set persistence to False
            if (e.affectsConfiguration(DatasetTree.persistenceSchema)) {
                const setting = Object.assign({}, vscode.workspace.getConfiguration().get(DatasetTree.persistenceSchema));
                if (!setting.persistence) {
                    setting.favorites = [];
                    setting.history = [];
                    yield vscode.workspace.getConfiguration().update(DatasetTree.persistenceSchema, setting, vscode.ConfigurationTarget.Global);
                }
            }
        });
    }
    addHistory(criteria) {
        return __awaiter(this, void 0, void 0, function* () {
            this.mHistory.addHistory(criteria);
            this.refresh();
        });
    }
    getHistory() {
        return this.mHistory.getHistory();
    }
    addRecall(criteria) {
        return __awaiter(this, void 0, void 0, function* () {
            this.mHistory.addRecall(criteria);
            this.refresh();
        });
    }
    getRecall() {
        return this.mHistory.getRecall();
    }
    removeRecall(name) {
        this.mHistory.removeRecall(name);
    }
    createFilterString(newFilter, node) {
        return __awaiter(this, void 0, void 0, function* () {
            // Store previous filters (before refreshing)
            let theFilter = this.getHistory()[0] || null;
            // Check if filter is currently applied
            if (node.pattern !== "" && theFilter) {
                const currentFilters = node.pattern.split(",");
                // Check if current filter includes the new node
                const matchedFilters = currentFilters.filter((filter) => {
                    const regex = new RegExp(filter.trim().replace(`*`, "") + "$");
                    return regex.test(newFilter);
                });
                if (matchedFilters.length === 0) {
                    // remove the last segement with a dot of the name for the new filter
                    theFilter = `${node.pattern},${newFilter}`;
                }
                else {
                    theFilter = node.pattern;
                }
            }
            else {
                // No filter is currently applied
                theFilter = newFilter;
            }
            return theFilter;
        });
    }
    /**
     * Opens a data set & reveals it in the tree
     *
     */
    openItemFromPath(itemPath, sessionNode) {
        return __awaiter(this, void 0, void 0, function* () {
            let parentNode = null;
            let memberNode;
            let parentName = null;
            let memberName = null;
            // Get node names from path
            if (itemPath.indexOf("(") > -1) {
                parentName = itemPath.substring(itemPath.indexOf(" ") + 1, itemPath.indexOf("(")).trim();
                memberName = itemPath.substring(itemPath.indexOf("(") + 1, itemPath.indexOf(")"));
            }
            else {
                parentName = itemPath.substring(itemPath.indexOf(" ") + 1);
            }
            // Update tree filter to include selected node, and expand session node in tree
            sessionNode.tooltip = sessionNode.pattern = yield this.createFilterString(parentName, sessionNode);
            sessionNode.collapsibleState = vscode.TreeItemCollapsibleState.Expanded;
            sessionNode.label = sessionNode.label.trim() + " ";
            sessionNode.label = sessionNode.label.trim();
            sessionNode.dirty = true;
            yield this.refresh();
            let children = yield sessionNode.getChildren();
            // Find parent node in tree
            parentNode = children.find((child) => child.label.trim() === parentName);
            if (parentNode) {
                parentNode.label = parentNode.tooltip = parentNode.pattern = parentName;
                parentNode.dirty = true;
            }
            else {
                vscode.window.showInformationMessage(localize(10, null));
                this.removeRecall(itemPath);
                return;
            }
            // If parent node has a child, expand parent node, and find child in tree
            if (itemPath.indexOf("(") > -1) {
                parentNode.collapsibleState = vscode.TreeItemCollapsibleState.Expanded;
                children = yield parentNode.getChildren();
                memberNode = children.find((child) => child.label.trim() === memberName);
                if (!memberNode) {
                    vscode.window.showInformationMessage(localize(11, null));
                    this.removeRecall(itemPath);
                    return;
                }
                else {
                    memberNode.getParent().label = memberNode.getParent().label.trim() + " ";
                    memberNode.getParent().label = memberNode.getParent().label.trim();
                    memberNode.getParent().collapsibleState = vscode.TreeItemCollapsibleState.Expanded;
                    this.addHistory(`${parentName}(${memberName})`);
                    dsActions.openPS(memberNode, true, this);
                }
            }
            else {
                this.addHistory(parentName);
                dsActions.openPS(parentNode, true, this);
            }
        });
    }
    searchInLoadedItems() {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.debug(localize(12, null));
            const loadedItems = [];
            const sessions = yield this.getChildren();
            // Add all data sets loaded in the tree to an array
            for (const session of sessions) {
                if (!session.contextValue.includes(globals.FAVORITE_CONTEXT)) {
                    if (session.children) {
                        for (const node of session.children) {
                            if (node.contextValue !== globals.INFORMATION_CONTEXT) {
                                loadedItems.push(node);
                                for (const member of node.children) {
                                    if (member.contextValue !== globals.INFORMATION_CONTEXT) {
                                        loadedItems.push(member);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            return loadedItems;
        });
    }
    datasetFilterPrompt(node) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.debug(localize(13, null));
            let pattern;
            yield this.checkCurrentProfile(node);
            if (Profiles_1.Profiles.getInstance().validProfile === 0) {
                if (node.contextValue === globals.DS_SESSION_CONTEXT) {
                    if (this.mHistory.getHistory().length > 0) {
                        const createPick = new utils_1.FilterDescriptor(DatasetTree.defaultDialogText);
                        const items = this.mHistory.getHistory().map((element) => new utils_1.FilterItem(element));
                        if (globals.ISTHEIA) {
                            const options1 = {
                                placeHolder: localize(14, null)
                            };
                            // get user selection
                            const choice = (yield vscode.window.showQuickPick([createPick, ...items], options1));
                            if (!choice) {
                                vscode.window.showInformationMessage(localize(15, null));
                                return;
                            }
                            pattern = choice === createPick ? "" : choice.label;
                        }
                        else {
                            const quickpick = vscode.window.createQuickPick();
                            quickpick.items = [createPick, ...items];
                            quickpick.placeholder = localize(16, null);
                            quickpick.ignoreFocusOut = true;
                            quickpick.show();
                            const choice = yield utils_1.resolveQuickPickHelper(quickpick);
                            quickpick.hide();
                            if (!choice) {
                                vscode.window.showInformationMessage(localize(17, null));
                                return;
                            }
                            if (choice instanceof utils_1.FilterDescriptor) {
                                if (quickpick.value) {
                                    pattern = quickpick.value;
                                }
                            }
                            else {
                                pattern = choice.label;
                            }
                        }
                    }
                    if (!pattern) {
                        // manually entering a search
                        const options2 = {
                            prompt: localize(18, null),
                            value: node.pattern,
                        };
                        // get user input
                        pattern = yield vscode.window.showInputBox(options2);
                        if (!pattern) {
                            vscode.window.showInformationMessage(localize(19, null));
                            return;
                        }
                    }
                }
                else {
                    // executing search from saved search in favorites
                    pattern = node.label.trim().substring(node.getLabel().indexOf(":") + 2);
                    const session = node.label.trim().substring(node.label.trim().indexOf("[") + 1, node.label.trim().indexOf("]"));
                    yield this.addSession(session);
                    const faveNode = node;
                    node = this.mSessionNodes.find((tempNode) => tempNode.label.trim() === session);
                    if ((!node.getSession().ISession.user) || (!node.getSession().ISession.password)) {
                        node.getSession().ISession.user = faveNode.getSession().ISession.user;
                        node.getSession().ISession.password = faveNode.getSession().ISession.password;
                        node.getSession().ISession.base64EncodedAuth = faveNode.getSession().ISession.base64EncodedAuth;
                    }
                }
                // update the treeview with the new pattern
                node.label = node.label.trim() + " ";
                node.tooltip = node.pattern = pattern.toUpperCase();
                node.collapsibleState = vscode.TreeItemCollapsibleState.Expanded;
                node.dirty = true;
                const icon = icons_1.getIconByNode(node);
                if (icon) {
                    node.iconPath = icon.path;
                }
                this.addHistory(node.pattern);
            }
        });
    }
    checkCurrentProfile(node) {
        return __awaiter(this, void 0, void 0, function* () {
            const profile = node.getProfile();
            yield Profiles_1.Profiles.getInstance().checkCurrentProfile(profile);
            yield this.refresh();
        });
    }
    /**
     * Rename data set member
     *
     * @param node - The node
     */
    renameDataSetMember(node) {
        return __awaiter(this, void 0, void 0, function* () {
            const beforeMemberName = node.label.trim();
            let dataSetName;
            let profileLabel;
            if (node.getParent().contextValue.includes(globals.FAV_SUFFIX)) {
                profileLabel = node.getParent().getLabel().substring(0, node.getParent().getLabel().indexOf(":") + 2);
                dataSetName = node.getParent().getLabel().substring(node.getParent().getLabel().indexOf(":") + 2);
            }
            else {
                dataSetName = node.getParent().getLabel();
            }
            const afterMemberName = yield vscode.window.showInputBox({ value: beforeMemberName });
            const beforeFullPath = utils_2.getDocumentFilePath(`${node.getParent().getLabel()}(${node.getLabel()})`, node);
            const closedOpenedInstance = yield dsActions.closeOpenedTextFile(beforeFullPath);
            this.log.debug(localize(20, null) + afterMemberName);
            if (afterMemberName) {
                try {
                    yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(node.getProfile()).renameDataSetMember(dataSetName, beforeMemberName, afterMemberName);
                    node.label = afterMemberName;
                }
                catch (err) {
                    this.log.error(localize(21, null) + JSON.stringify(err));
                    yield utils_1.errorHandling(err, profileLabel, localize(22, null) + err.message);
                    throw err;
                }
                let otherParent;
                if (node.getParent().contextValue.includes(globals.FAV_SUFFIX)) {
                    otherParent = this.findNonFavoritedNode(node.getParent());
                }
                else {
                    otherParent = this.findFavoritedNode(node.getParent());
                }
                if (otherParent) {
                    const otherMember = otherParent.children.find((child) => child.label === beforeMemberName);
                    if (otherMember) {
                        otherMember.label = afterMemberName;
                        this.refreshElement(otherMember);
                    }
                }
                this.refreshElement(node);
                if (fs.existsSync(beforeFullPath)) {
                    fs.unlinkSync(beforeFullPath);
                }
                if (closedOpenedInstance) {
                    vscode.commands.executeCommand("zowe.ZoweNode.openPS", node);
                }
            }
        });
    }
    /**
     * Rename data set
     *
     * @param node - The node
     */
    renameDataSet(node) {
        return __awaiter(this, void 0, void 0, function* () {
            let beforeDataSetName = node.label.trim();
            let favPrefix = "";
            let isFavourite;
            if (node.contextValue.includes(globals.FAV_SUFFIX)) {
                isFavourite = true;
                favPrefix = node.label.substring(0, node.label.indexOf(":") + 2);
                beforeDataSetName = node.label.substring(node.label.indexOf(":") + 2);
            }
            const afterDataSetName = yield vscode.window.showInputBox({ value: beforeDataSetName });
            const beforeFullPath = utils_2.getDocumentFilePath(node.getLabel(), node);
            const closedOpenedInstance = yield dsActions.closeOpenedTextFile(beforeFullPath);
            this.log.debug(localize(23, null) + afterDataSetName);
            if (afterDataSetName) {
                try {
                    yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(node.getProfile()).renameDataSet(beforeDataSetName, afterDataSetName);
                    node.label = `${favPrefix}${afterDataSetName}`;
                    if (isFavourite) {
                        const profile = favPrefix.substring(1, favPrefix.indexOf("]"));
                        this.renameNode(profile, beforeDataSetName, afterDataSetName);
                    }
                    else {
                        const temp = node.label;
                        node.label = "[" + node.getSessionNode().label.trim() + "]: " + beforeDataSetName;
                        this.renameFavorite(node, afterDataSetName);
                        node.label = temp;
                    }
                    this.refreshElement(node);
                    this.updateFavorites();
                    if (fs.existsSync(beforeFullPath)) {
                        fs.unlinkSync(beforeFullPath);
                    }
                    if (closedOpenedInstance) {
                        vscode.commands.executeCommand("zowe.ZoweNode.openPS", node);
                    }
                }
                catch (err) {
                    this.log.error(localize(24, null) + JSON.stringify(err));
                    yield utils_1.errorHandling(err, favPrefix, localize(25, null) + err.message);
                    throw err;
                }
            }
        });
    }
    /**
     * Adds a single session to the data set tree
     *
     */
    addSingleSession(profile) {
        if (profile) {
            // If session is already added, do nothing
            if (this.mSessionNodes.find((tempNode) => tempNode.label.trim() === profile.name)) {
                return;
            }
            // Uses loaded profile to create a session with the MVS API
            const session = ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(profile).getSession();
            // Creates ZoweDatasetNode to track new session and pushes it to mSessionNodes
            const node = new ZoweDatasetNode_1.ZoweDatasetNode(profile.name, vscode.TreeItemCollapsibleState.Collapsed, null, session, undefined, undefined, profile);
            node.contextValue = globals.DS_SESSION_CONTEXT;
            const icon = icons_1.getIconByNode(node);
            if (icon) {
                node.iconPath = icon.path;
            }
            this.mSessionNodes.push(node);
            this.mHistory.addSession(profile.name);
        }
    }
}
exports.DatasetTree = DatasetTree;
DatasetTree.persistenceSchema = "Zowe-DS-Persistent";
DatasetTree.defaultDialogText = "\uFF0B " + localize(26, null);

//# sourceMappingURL=DatasetTree.js.map
