"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const globals = require("../globals");
const zowe = require("@zowe/cli");
const utils_1 = require("../utils");
const utils_2 = require("../shared/utils");
const Profiles_1 = require("../Profiles");
const ZoweExplorerApiRegister_1 = require("../api/ZoweExplorerApiRegister");
const nls = require("vscode-nls");
const SpoolProvider_1 = require("../SpoolProvider");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
/**
 * Refresh all jobs in the job tree
 *
 * @param jobsProvider The tree to refresh
 */
function refreshAllJobs(jobsProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        yield Profiles_1.Profiles.getInstance().refresh();
        jobsProvider.mSessionNodes.forEach((jobNode) => {
            if (jobNode.contextValue === globals.JOBS_SESSION_CONTEXT) {
                utils_2.labelHack(jobNode);
                jobNode.children = [];
                jobNode.dirty = true;
                utils_2.refreshTree(jobNode);
            }
        });
        yield jobsProvider.refresh();
    });
}
exports.refreshAllJobs = refreshAllJobs;
/**
 * Download all the spool content for the specified job.
 *
 * @param job The job to download the spool content from
 */
function downloadSpool(job) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const dirUri = yield vscode.window.showOpenDialog({
                openLabel: localize(0, null),
                canSelectFolders: true,
                canSelectFiles: false,
                canSelectMany: false
            });
            if (dirUri !== undefined) {
                ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getJesApi(job.getProfile()).downloadSpoolContent({
                    jobid: job.job.jobid,
                    jobname: job.job.jobname,
                    outDir: dirUri[0].fsPath
                });
            }
        }
        catch (error) {
            yield utils_1.errorHandling(error, null, error.message);
        }
    });
}
exports.downloadSpool = downloadSpool;
/**
 * Download the spool content for the specified job
 *
 * @param session The session to which the job belongs
 * @param spool The IJobFile to get the spool content for
 */
function getSpoolContent(jobsProvider, session, spool) {
    return __awaiter(this, void 0, void 0, function* () {
        const zosmfProfile = Profiles_1.Profiles.getInstance().loadNamedProfile(session);
        // This has a direct access to Profiles checkcurrentProfile() because I am able to get the profile now.
        yield Profiles_1.Profiles.getInstance().checkCurrentProfile(zosmfProfile);
        if (Profiles_1.Profiles.getInstance().validProfile === 0) {
            try {
                const uri = SpoolProvider_1.encodeJobFile(session, spool);
                const document = yield vscode.workspace.openTextDocument(uri);
                yield vscode.window.showTextDocument(document);
            }
            catch (error) {
                yield utils_1.errorHandling(error, session, error.message);
            }
        }
    });
}
exports.getSpoolContent = getSpoolContent;
/**
 * Refresh a node in the job tree
 *
 * @param node The node to refresh
 * @param jobsProvider The tree to which the refreshed node belongs
 */
function refreshJobsServer(node, jobsProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        let sesNamePrompt;
        if (node.contextValue.endsWith(globals.FAV_SUFFIX)) {
            sesNamePrompt = node.label.substring(1, node.label.indexOf("]"));
        }
        else {
            sesNamePrompt = node.label;
        }
        jobsProvider.checkCurrentProfile(node);
        if (Profiles_1.Profiles.getInstance().validProfile === 0) {
            yield jobsProvider.refreshElement(node);
        }
    });
}
exports.refreshJobsServer = refreshJobsServer;
/**
 * Download the JCL content for the specified job.
 *
 * @param job The job to download the JCL content from
 */
function downloadJcl(job) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const jobJcl = yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getJesApi(job.getProfile()).getJclForJob(job.job);
            const jclDoc = yield vscode.workspace.openTextDocument({ language: "jcl", content: jobJcl });
            yield vscode.window.showTextDocument(jclDoc);
        }
        catch (error) {
            yield utils_1.errorHandling(error, null, error.message);
        }
    });
}
exports.downloadJcl = downloadJcl;
/**
 * Modify a job command
 *
 * @param job The job on which to modify a command
 */
function modifyCommand(job) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const command = yield vscode.window.showInputBox({ prompt: localize(1, null) });
            if (command !== undefined) {
                const response = yield zowe.IssueCommand.issueSimple(job.getSession(), `f ${job.job.jobname},${command}`);
                vscode.window.showInformationMessage(localize(2, null) + response.commandResponse);
            }
        }
        catch (error) {
            yield utils_1.errorHandling(error, null, error.message);
        }
    });
}
exports.modifyCommand = modifyCommand;
/**
 * Stop a job command
 *
 * @param job The job on which to stop a command
 */
function stopCommand(job) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const response = yield zowe.IssueCommand.issueSimple(job.getSession(), `p ${job.job.jobname}`);
            vscode.window.showInformationMessage(localize(3, null) + response.commandResponse);
        }
        catch (error) {
            yield utils_1.errorHandling(error, null, error.message);
        }
    });
}
exports.stopCommand = stopCommand;
/**
 * Set the owner of a job
 *
 * @param job The job to set the owner of
 * @param jobsProvider The tree to which the updated node belongs
 */
function setOwner(job, jobsProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const newOwner = yield vscode.window.showInputBox({ prompt: localize(4, null) });
        job.owner = newOwner;
        jobsProvider.refreshElement(job);
    });
}
exports.setOwner = setOwner;
/**
 * Set the prefix of a job
 *
 * @param job The job to set the prefix of
 * @param jobsProvider The tree to which the updated node belongs
 */
function setPrefix(job, jobsProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const newPrefix = yield vscode.window.showInputBox({ prompt: localize(5, null) });
        job.prefix = newPrefix;
        jobsProvider.refreshElement(job);
    });
}
exports.setPrefix = setPrefix;

//# sourceMappingURL=actions.js.map
