"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const fs = require("fs");
const globals = require("../globals");
const path = require("path");
const utils_1 = require("../shared/utils");
const utils_2 = require("../utils");
const Profiles_1 = require("../Profiles");
const ZoweExplorerApiRegister_1 = require("../api/ZoweExplorerApiRegister");
const isbinaryfile_1 = require("isbinaryfile");
const nls = require("vscode-nls");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
/**
 * Prompts the user for a path, and populates the [TreeView]{@link vscode.TreeView} based on the path
 *
 * @param {ZoweUSSNode} node - The session node
 * @param {ussTree} ussFileProvider - Current ussTree used to populate the TreeView
 * @returns {Promise<void>}
 */
function createUSSNode(node, ussFileProvider, nodeType, isTopLevel) {
    return __awaiter(this, void 0, void 0, function* () {
        const name = yield vscode.window.showInputBox({
            placeHolder: localize(0, null)
        });
        if (name) {
            try {
                const filePath = `${node.fullPath}/${name}`;
                yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(node.getProfile()).create(filePath, nodeType);
                if (isTopLevel) {
                    refreshAllUSS(ussFileProvider);
                }
                else {
                    ussFileProvider.refreshElement(node);
                }
            }
            catch (err) {
                utils_2.errorHandling(err, node.mProfileName, localize(1, null) + err.message);
                throw (err);
            }
            ussFileProvider.refresh();
        }
    });
}
exports.createUSSNode = createUSSNode;
function refreshUSSInTree(node, ussFileProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        yield ussFileProvider.refreshElement(node);
    });
}
exports.refreshUSSInTree = refreshUSSInTree;
function createUSSNodeDialog(node, ussFileProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        yield ussFileProvider.checkCurrentProfile(node);
        if (Profiles_1.Profiles.getInstance().validProfile === 0) {
            const quickPickOptions = {
                placeHolder: `What would you like to create at ${node.fullPath}?`,
                ignoreFocusOut: true,
                canPickMany: false
            };
            const type = yield vscode.window.showQuickPick([globals.USS_DIR_CONTEXT, "File"], quickPickOptions);
            const isTopLevel = true;
            return createUSSNode(node, ussFileProvider, type, isTopLevel);
        }
    });
}
exports.createUSSNodeDialog = createUSSNodeDialog;
/**
 * Refreshes treeView
 *
 * @param {USSTree} ussFileProvider
 */
function refreshAllUSS(ussFileProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        yield Profiles_1.Profiles.getInstance().refresh();
        ussFileProvider.mSessionNodes.forEach((sessNode) => {
            if (sessNode.contextValue === globals.USS_SESSION_CONTEXT) {
                utils_1.labelHack(sessNode);
                sessNode.children = [];
                sessNode.dirty = true;
                utils_1.refreshTree(sessNode);
            }
        });
        yield ussFileProvider.refresh();
    });
}
exports.refreshAllUSS = refreshAllUSS;
/**
 * Process for renaming a USS Node. This could be a Favorite Node
 *
 * @param {IZoweTreeNode} originalNode
 * @param {USSTree} ussFileProvider
 * @param {string} filePath
 */
function renameUSSNode(originalNode, ussFileProvider, filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        // Could be a favorite or regular entry always deal with the regular entry
        const isFav = originalNode.contextValue.endsWith(globals.FAV_SUFFIX);
        const oldLabel = originalNode.label;
        const parentPath = originalNode.fullPath.substr(0, originalNode.fullPath.indexOf(oldLabel));
        // Check if an old favorite exists for this node
        const oldFavorite = isFav ? originalNode : ussFileProvider.mFavorites.find((temp) => (temp.shortLabel === oldLabel) && (temp.fullPath.substr(0, temp.fullPath.indexOf(oldLabel)) === parentPath));
        const newName = yield vscode.window.showInputBox({ value: oldLabel });
        if (newName && newName !== oldLabel) {
            try {
                let newNamePath = path.join(parentPath + newName);
                newNamePath = newNamePath.replace(/\\/g, "/"); // Added to cover Windows backslash issue
                yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(originalNode.getProfile()).rename(originalNode.fullPath, newNamePath);
                yield deleteFromDisk(originalNode, filePath);
                originalNode.rename(newNamePath);
                if (oldFavorite) {
                    ussFileProvider.removeFavorite(oldFavorite);
                    oldFavorite.rename(newNamePath);
                    ussFileProvider.addFavorite(oldFavorite);
                }
            }
            catch (err) {
                utils_2.errorHandling(err, originalNode.mProfileName, localize(2, null) + err.message);
                throw (err);
            }
        }
    });
}
exports.renameUSSNode = renameUSSNode;
/**
 * Marks file as deleted from disk
 *
 * @param {ZoweUSSNode} node
 */
function deleteFromDisk(node, filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            if (fs.existsSync(filePath)) {
                fs.unlinkSync(filePath);
            }
        }
        // tslint:disable-next-line: no-empty
        catch (err) {
        }
    });
}
exports.deleteFromDisk = deleteFromDisk;
function uploadDialog(node, ussFileProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const fileOpenOptions = {
            canSelectFiles: true,
            openLabel: "Upload Files",
            canSelectMany: true
        };
        const value = yield vscode.window.showOpenDialog(fileOpenOptions);
        yield Promise.all(value.map((item) => __awaiter(this, void 0, void 0, function* () {
            const isBinary = isbinaryfile_1.isBinaryFileSync(item.fsPath);
            if (isBinary) {
                yield uploadBinaryFile(node, item.fsPath);
            }
            else {
                const doc = yield vscode.workspace.openTextDocument(item);
                yield uploadFile(node, doc);
            }
        })));
        ussFileProvider.refresh();
    });
}
exports.uploadDialog = uploadDialog;
function uploadBinaryFile(node, filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const localFileName = path.parse(filePath).base;
            const ussName = `${node.fullPath}/${localFileName}`;
            yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(node.getProfile()).putContents(filePath, ussName, true);
        }
        catch (e) {
            utils_2.errorHandling(e, node.mProfileName, e.message);
        }
    });
}
exports.uploadBinaryFile = uploadBinaryFile;
function uploadFile(node, doc) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const localFileName = path.parse(doc.fileName).base;
            const ussName = `${node.fullPath}/${localFileName}`;
            yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(node.getProfile()).putContents(doc.fileName, ussName);
        }
        catch (e) {
            utils_2.errorHandling(e, node.mProfileName, e.message);
        }
    });
}
exports.uploadFile = uploadFile;
/**
 * Copies full path for the selected Zowe USS node
 *
 * @param {ZoweUSSNode} node
 */
function copyPath(node) {
    return __awaiter(this, void 0, void 0, function* () {
        if (globals.ISTHEIA) {
            // Remove when Theia supports VS Code API for accessing system clipboard
            vscode.window.showInformationMessage(localize(3, null));
            return;
        }
        vscode.env.clipboard.writeText(node.fullPath);
    });
}
exports.copyPath = copyPath;
/**
 * Switch the download type and redownload the file.
 *
 * @param node The file that is going to be downloaded
 * @param binary Whether the file should be downloaded as binary or not
 * @param ussFileProvider Our USSTree object
 */
function changeFileType(node, binary, ussFileProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        node.setBinary(binary);
        yield node.openUSS(true, true, ussFileProvider);
        ussFileProvider.refresh();
    });
}
exports.changeFileType = changeFileType;
/**
 * Uploads the file to the mainframe
 *
 * @export
 * @param {Session} session - Desired session
 * @param {vscode.TextDocument} doc - TextDocument that is being saved
 */
function saveUSSFile(doc, ussFileProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        globals.LOG.debug(localize(4, null) + doc.fileName);
        const start = path.join(globals.USS_DIR + path.sep).length;
        const ending = doc.fileName.substring(start);
        const sesName = ending.substring(0, ending.indexOf(path.sep));
        const remote = ending.substring(sesName.length).replace(/\\/g, "/");
        // get session from session name
        let documentSession;
        let binary;
        let node;
        // TODO remove as
        const sesNode = (ussFileProvider.mSessionNodes.find((child) => child.getProfileName() && child.getProfileName() === sesName.trim()));
        if (sesNode) {
            documentSession = sesNode.getSession();
            binary = Object.keys(sesNode.binaryFiles).find((child) => child === remote) !== undefined;
        }
        // Get specific node based on label and parent tree (session / favorites)
        let nodes;
        if (!sesNode || sesNode.children.length === 0) {
            // saving from favorites
            nodes = utils_1.concatChildNodes(ussFileProvider.mFavorites);
        }
        else {
            // saving from session
            nodes = utils_1.concatChildNodes([sesNode]);
        }
        node = nodes.find((zNode) => {
            if (zNode.contextValue === globals.DS_FAV_TEXT_FILE_CONTEXT || zNode.contextValue === globals.DS_TEXT_FILE_CONTEXT) {
                return (zNode.fullPath.trim() === remote);
            }
            else {
                return false;
            }
        });
        // define upload options
        let etagToUpload;
        let returnEtag;
        if (node) {
            etagToUpload = node.getEtag();
            if (etagToUpload) {
                returnEtag = true;
            }
        }
        try {
            const uploadResponse = yield vscode.window.withProgress({
                location: vscode.ProgressLocation.Notification,
                title: localize(5, null)
            }, () => {
                return ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(sesNode.getProfile()).putContents(doc.fileName, remote, binary, null, etagToUpload, returnEtag); // TODO MISSED TESTING
            });
            if (uploadResponse.success) {
                vscode.window.showInformationMessage(uploadResponse.commandResponse);
                // set local etag with the new etag from the updated file on mainframe
                node.setEtag(uploadResponse.apiResponse.etag);
                // this part never runs! zowe.Upload.fileToUSSFile doesn't return success: false, it just throws the error which is caught below!!!!!
            }
            else {
                vscode.window.showErrorMessage(uploadResponse.commandResponse);
            }
        }
        catch (err) {
            // TODO: error handling must not be zosmf specific
            if (err.message.includes(localize(6, null))) {
                // Store old document text in a separate variable, to be used on merge conflict
                const oldDocText = doc.getText();
                const oldDocLineCount = doc.lineCount;
                const downloadResponse = yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(node.getProfile()).getContents(node.fullPath, {
                    file: node.getUSSDocumentFilePath(),
                    binary,
                    returnEtag: true
                });
                // re-assign etag, so that it can be used with subsequent requests
                const downloadEtag = downloadResponse.apiResponse.etag;
                if (downloadEtag !== etagToUpload) {
                    node.setEtag(downloadEtag);
                }
                this.downloaded = true;
                vscode.window.showWarningMessage(localize(7, null));
                const startPosition = new vscode.Position(0, 0);
                const endPosition = new vscode.Position(oldDocLineCount, 0);
                const deleteRange = new vscode.Range(startPosition, endPosition);
                yield vscode.window.activeTextEditor.edit((editBuilder) => {
                    // re-write the old content in the editor view
                    editBuilder.delete(deleteRange);
                    editBuilder.insert(startPosition, oldDocText);
                });
                yield vscode.window.activeTextEditor.document.save();
            }
            else {
                globals.LOG.error(localize(8, null) + JSON.stringify(err));
                yield utils_2.errorHandling(err, sesName, err.message);
            }
        }
    });
}
exports.saveUSSFile = saveUSSFile;

//# sourceMappingURL=actions.js.map
